package com.example.aspdemo;

import android.content.Context;
import android.net.Uri;
import android.util.JsonReader;
import android.util.Log;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;

import java.io.IOException;
import java.io.StringReader;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.TimeZone;

public class ADHelper {
    private final static String TAG = "ADHelper";

    private static Object sObjLocker = new Object();
    private static ADHelper sInstance = null;

    private String mRegionId = null;
    private String mRegionNumber = null;
    private String mLoginToken = null;
    private String mSessionId = null;
    BigInteger mRandClientId = null;

    public interface IADLoginResponse {
        void onSuccess(String response);
        void onFailed(long errorCode);
    }

    public interface IADDesktopListResponse {
        void onSuccess(List<Desktop> desktopList);
        void onFailed(long errorCode);
    }

    public interface IADConnectionTicketResponse {
        void onSuccess(String ticket);
        void onFailed(long errorCode);
    }

    public class Desktop {
        public String mId;
        public String mName;
        public String mOSType;
        public String mStatus;
    }

    private ADHelper() {
        mRandClientId = new BigInteger(32, new Random());
    }

    static public ADHelper getInstance() {
        synchronized (sObjLocker) {
            if (sInstance == null) {
                sInstance = new ADHelper();
            }
        }

        return sInstance;
    }

    public void setRegionId(String regionId) {
        mRegionId = regionId;
    }

    public void setRegionNumber(String regionNumber) {
        mRegionNumber = regionNumber;
    }

    public void getConnectionTicket(Context context, Desktop desktop, final IADConnectionTicketResponse adConnectionTicketResposne) {
        getConnectionTicket(context, desktop, null, 0, adConnectionTicketResposne);
    }

    public void getDesktopList(Context context, final IADDesktopListResponse adDesktopListResponse) {
        RequestQueue queue = Volley.newRequestQueue(context);

        String url = populateBaseADUri()
                .appendQueryParameter("Action", "DescribeDesktops")
                .appendQueryParameter("LoginToken", mLoginToken)
                .appendQueryParameter("MaxResults", "100")
                .appendQueryParameter("NextToken", "")
                .appendQueryParameter("ProtocolType", "both")
                .appendQueryParameter("SessionId", mSessionId)
                .build().toString();

        Log.i(TAG, url);

        StringRequest stringRequest = new StringRequest(Request.Method.GET, url,
                new Response.Listener<String>() {
                    @Override
                    public void onResponse(String response) {
                        Log.i(TAG, response);
                        ArrayList<Desktop> desktops = new ArrayList<>();

                        JsonReader reader = new JsonReader(new StringReader(response));
                        try {
                            reader.beginObject();
                            while (reader.hasNext()) {
                                final String name = reader.nextName();
                                if (name.equals("Desktops")) {
                                    reader.beginArray();
                                    while (reader.hasNext()) {
                                        Desktop desktop = new Desktop();
                                        reader.beginObject();
                                        while (reader.hasNext()) {
                                            final String n = reader.nextName();
                                            if (n.equals("DesktopName")) {
                                                desktop.mName = reader.nextString();
                                            } else if (n.equals("DesktopId")) {
                                                desktop.mId = reader.nextString();
                                            } else if (n.equals("DesktopStatus")) {
                                                desktop.mStatus = reader.nextString();
                                            } else if (n.equals("OsType")) {
                                                desktop.mOSType = reader.nextString();
                                            } else {
                                                reader.skipValue();
                                            }
                                        }
                                        desktops.add(desktop);
                                        reader.endObject();
                                    }
                                    reader.endArray();
                                } else {
                                    reader.skipValue();
                                }
                            }
                        } catch (IOException e) {
                            e.printStackTrace();
                        }

                        adDesktopListResponse.onSuccess(desktops);
                    }
                }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                if (error != null && error.networkResponse != null) {
                    Log.i(TAG, "error code: " + error.networkResponse.statusCode);
                    adDesktopListResponse.onFailed(error.networkResponse.statusCode);
                } else {
                    adDesktopListResponse.onFailed(-1);
                }
            }
        });

        // Add the request to the RequestQueue.
        queue.add(stringRequest);
    }

    public void login(String username, String password, Context context, final IADLoginResponse adLoginResponse) {
        RequestQueue queue = Volley.newRequestQueue(context);

        String url = populateBaseADUri()
                .appendQueryParameter("Action", "GetLoginToken")
                .appendQueryParameter("CurrentStage", "ADPassword")
                .appendQueryParameter("EndUserId", username)
                .appendQueryParameter("Password", password)
                .build().toString();

        Log.i(TAG, url);

        StringRequest stringRequest = new StringRequest(Request.Method.GET, url,
                new Response.Listener<String>() {
                    @Override
                    public void onResponse(String response) {
                        Log.i(TAG, response);
                        JsonReader reader = new JsonReader(new StringReader(response));
                        try {
                            reader.beginObject();
                            while (reader.hasNext()) {
                                final String name = reader.nextName();
                                if (name.equals("LoginToken")) {
                                    mLoginToken = reader.nextString();
                                } else if (name.equals("SessionId")) {
                                    mSessionId = reader.nextString();
                                } else {
                                    reader.skipValue();
                                }
                            }
                        } catch (IOException e) {
                            e.printStackTrace();
                        }

                        adLoginResponse.onSuccess(response);
                    }
                }, new Response.ErrorListener() {
                    @Override
                    public void onErrorResponse(VolleyError error) {
                        Log.i(TAG, "error code: " + error.networkResponse.statusCode);
                        adLoginResponse.onFailed(error.networkResponse.statusCode);
                    }
        });

        // Add the request to the RequestQueue.
        queue.add(stringRequest);
    }

    private void getConnectionTicket(Context context, Desktop desktop, final String taskId,
                                     final int currentLevel,
                                     final IADConnectionTicketResponse adConnectionTicketResposne) {
        RequestQueue queue = Volley.newRequestQueue(context);

        String url = populateBaseADUri()
                    .appendQueryParameter("Action", "GetConnectionTicket")
                    .appendQueryParameter("LoginToken", mLoginToken)
                    .appendQueryParameter("DesktopId", desktop.mId)
                    .appendQueryParameter("SessionId", mSessionId)
                    .appendQueryParameter("TaskId", taskId == null ? "" : taskId)
                    .build().toString();

        Log.i(TAG, url);

        StringRequest stringRequest = new StringRequest(Request.Method.GET, url,
                new Response.Listener<String>() {
                    @Override
                    public void onResponse(String response) {
                        Log.i(TAG, response);
                        JsonReader reader = new JsonReader(new StringReader(response));
                        boolean retry = false;
                        String nextTaskId = taskId;
                        int nextLevel = currentLevel + 1;
                        String taskStatus = null;
                        String connectionTicket = null;
                        try {
                            reader.beginObject();
                            while (reader.hasNext()) {
                                final String name = reader.nextName();
                                if (name.equals("TaskId")) {
                                    nextTaskId = reader.nextString();
                                } else if (name.equals("TaskStatus")) {
                                    taskStatus = reader.nextString();
                                    retry = nextLevel < 10 && !taskStatus.equals("FINISHED");
                                } else if (name.equals("Ticket")) {
                                    connectionTicket = reader.nextString();
                                } else {
                                    reader.skipValue();
                                }
                            }
                        } catch (IOException e) {
                            e.printStackTrace();
                        }

                        if (retry) {
                            getConnectionTicket(context, desktop, nextTaskId, nextLevel, adConnectionTicketResposne);
                        } else if (taskStatus == null || !taskStatus.equals("FINISHED") || connectionTicket == null) {
                            Log.e(TAG, "Cannot get connection ticket after 10 retries");
                            adConnectionTicketResposne.onFailed(Long.MAX_VALUE);
                        } else {

                            adConnectionTicketResposne.onSuccess(connectionTicket);
                        }
                    }
                }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                if (error != null && error.networkResponse != null) {
                    Log.i(TAG, "error code: " + error.networkResponse.statusCode);
                    adConnectionTicketResposne.onFailed(error.networkResponse.statusCode);
                } else {
                    adConnectionTicketResposne.onFailed(-1);
                }
            }
        });

        // Add the request to the RequestQueue.
        queue.add(stringRequest);
    }

    private Uri.Builder populateBaseADUri() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
        sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
        String currentDateTimeString = sdf.format(new Date());
        return Uri.parse("https://ecd." + mRegionId + ".aliyuncs.com/?")
                .buildUpon()
                .appendQueryParameter("ClientId", "Client" + mRandClientId)
                .appendQueryParameter("ClientOS", "Android 10")
                .appendQueryParameter("ClientVersion", "1.0.0")
                .appendQueryParameter("DirectoryId", mRegionId + "+dir-" + mRegionNumber)
                .appendQueryParameter("Format", "json")
                .appendQueryParameter("RegionId", mRegionId)
                .appendQueryParameter("SignatureNonce",
                        Calendar.getInstance(TimeZone.getTimeZone("GMT")).getTimeInMillis() + "" + (long)Math.ceil(Math.random() * 10000))
                .appendQueryParameter("Timestamp", currentDateTimeString)
                .appendQueryParameter("Version", "2020-10-02")
                .appendQueryParameter("product", "ecs_ad");
    }
}
