package com.example.aspdemo;

import java.nio.ByteBuffer;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.util.Log;
import android.view.InputDevice;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.PointerIcon;
import android.view.SurfaceView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

import com.aliyun.wuying.aspsdk.aspengine.IASPEngine;
import com.aliyun.wuying.aspsdk.aspengine.IASPEngineListener;
import com.aliyun.wuying.aspsdk.aspengine.ICursorListener;
import com.aliyun.wuying.aspsdk.aspengine.IOrientationUpdateListener;
import com.aliyun.wuying.aspsdk.aspengine.IResolutionUpdateListener;

public class ASPEngineDemoActivity extends Activity
        implements ICursorListener, IResolutionUpdateListener, IOrientationUpdateListener {
    static final String TAG = "ASPEngineDemoActivity";

    private String mHost = "30.224.192.130";
    private String mPort = "20082";
    private String mCAFilePath = null;
    private String mToken = null;

    private boolean mEnableTls = false;
    private boolean mEnableVDAgentCheck = true;
    private boolean mUseRTC = true;
    private boolean mEnablePerfStat = false;

    private SurfaceView mSurfaceView = null;

    private IASPEngine.Builder mEngineBuilder = null;
    private IASPEngine mEngine = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);

        setContentView(R.layout.activity_aspengine_demo);

        DialogHelper.getInstance().initialize(this);

        Bundle b = getIntent().getExtras();
        mHost = b.getString("host");
        mPort = b.getString("port");
        mToken = b.getString("token");
        mCAFilePath = b.getString("ca");
        mEnableTls = b.getBoolean("tls", false);
        mEnableVDAgentCheck = b.getBoolean("vdagent", true);
        mUseRTC = b.getBoolean("rtc", true);
        mEnablePerfStat = b.getBoolean("perf_stat", false);

        mSurfaceView = findViewById(R.id.asp_surface_view);
        mSurfaceView.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                if (mEngine != null) {
                    return mEngine.sendTouchEvent(motionEvent);
                }
                return false;
            }
        });

        mSurfaceView.setOnGenericMotionListener(new View.OnGenericMotionListener(){
            @Override
            public boolean onGenericMotion(View v, MotionEvent motionEvent) {
                Log.d(TAG,String.format("mouse, onGenericMotion: %d", motionEvent.getSource()));
                if (0 != (motionEvent.getSource() & InputDevice.SOURCE_MOUSE)) {
                    if (mEngine != null) {
                        return mEngine.sendMouseEvent(motionEvent);
                    }
                }
                return true;
            }
        });

        mEngineBuilder = new IASPEngine.Builder();
        mEngine = mEngineBuilder.enableVDAgentCheck(mEnableVDAgentCheck)
                .enableRTC(mUseRTC)
                .build();

        mEngine.setSurface(mSurfaceView);
        mEngine.enableStatistics(mEnablePerfStat);

        IASPEngineListener listener = new IASPEngineListener() {
            @Override
            public void onConnectionSuccess() {
                Log.i(TAG, "onConnectionSuccess");
                DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTED);
            }

            @Override
            public void onConnectionFailure(int errorCode, String errorMsg) {
                Log.i(TAG, "onConnectionFailure errorCode:" + errorCode + " errorMsg:" + errorMsg);
                DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTION_ERROR,
                        String.valueOf(errorCode), errorMsg);
            }

            @Override
            public void onEngineError(int errorCode, String errorMsg) {
                Log.i(TAG, "onEngineError errorCode:" + errorCode + " errorMsg:" + errorMsg);
                DialogHelper.getInstance().setState(DialogHelper.State.STATE_ENGINE_ERROR,
                        String.valueOf(errorCode), errorMsg);
            }

            @Override
            public void onDisconnected(int reason) {
                Log.i(TAG, "onDisconnected reason=" + reason);
                DialogHelper.getInstance().setState(DialogHelper.State.STATE_DISCONNECTED);
            }

            @Override
            public void onReconnect() {
                Log.i(TAG, "onReconnect");
                DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTING);
            }

            @Override
            public void onFirstFrameRendered(long timeCostMS) {
                Log.i(TAG, "onFirstFrameRendered");
            }

            @Override
            public void onPolicyUpdate(String policy) {
                Log.i(TAG, "policy=" + policy);
            }

            @Override
            public void onSessionSuccess() {

            }
        };

        mEngine.registerASPEngineListener(listener);
        mEngine.registerCursorListener(this);
        mEngine.registerResolutionUpdateListener(this);
        mEngine.registerOrientationUpdateListener(this);

        mEngine.start(mHost, mPort, "", mCAFilePath, mToken, mEnableTls);

        Log.d(TAG, "The CA certification path: " + mCAFilePath);

        DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTING);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mEngine != null) {
            DialogHelper.getInstance().reset();
            mEngine.stop();
            mEngine = null;
        }
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        if (mEngine != null) {
            if (!mEngine.sendKeyboardEvent(event)) {
                return super.dispatchKeyEvent(event);
            }
            return true;
        }
        return super.dispatchKeyEvent(event);
    }
    
    @Override
    public void onMonitorsDpiConfig(int a,int b) {

    }

    /*
     * Cursor callback BEGIN
     * */
    @Override
    public void onCursorBitmapUpdate(int hotX, int hotY, int width, int height, byte[] rgba, float scale) {
        Log.d(TAG,String.format("onCursorBitmapUpdate: hotX:%d, hotY:%d, width:%d, height:%d, len:%d :", hotX, hotY, width, height,rgba.length));
        //BitmapFactory.Options options = new BitmapFactory.Options();
        //options.inMutable = true;
        Bitmap rawBmp = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
        rawBmp.copyPixelsFromBuffer(ByteBuffer.wrap(rgba));
        if(rawBmp == null) {
            Log.i(TAG, "can't create cursor bitmap from rgba");
            return;
        }

        PointerIcon cursor = PointerIcon.create(rawBmp, hotX, hotY);
        mSurfaceView.setPointerIcon(cursor);
    }

    @Override
    public void onCursorHide()
    {
        Log.i(TAG, "onCursorHide");
        PointerIcon cursor = PointerIcon.getSystemIcon(getApplicationContext(),PointerIcon.TYPE_NULL);
        mSurfaceView.setPointerIcon(cursor);
    }

    @Override
    public void onCursorReset() {
        Log.i(TAG, "onCursorReset");
        PointerIcon cursor = PointerIcon.getSystemIcon(getApplicationContext(),PointerIcon.TYPE_DEFAULT);
        mSurfaceView.setPointerIcon(cursor);
    }

    @Override
    public void onCursorMove(int x, int y) {

    }
    /* Cursor callback END */

    @Override
    public void onResolutionUpdate(int oldWidth, int oldHeight, int width, int height) {
        Log.i(TAG, "onResolutionUpdate stream ow " + oldWidth + " oh " + oldHeight
                + " w " + width + " h " + height);
        Log.i(TAG, "current view width " + mSurfaceView.getWidth()
                + " height " + mSurfaceView.getHeight());

        float factorX = 1f;
        float factorY = 1f;

        float aspectRatio  = 1.0f * width / height;
        float viewAspectRatio = 1.0f * mSurfaceView.getWidth() / mSurfaceView.getHeight();
        Log.i(TAG, "StreamView aspect ratio " + viewAspectRatio
                + " VS remote window aspect ratio " + aspectRatio);
        // TODO: suppose current device orientation is landscape
        if (viewAspectRatio > aspectRatio) {
            // Align stream content with view height for landscape
            int expectedWidth = (int) (mSurfaceView.getHeight() * aspectRatio);
            Log.i(TAG, "Expect width " + expectedWidth);
            factorX = 1.0f * expectedWidth / mSurfaceView.getWidth();
        } else {
            // Align stream content with view width for portrait
            int expectedHeight = (int) (mSurfaceView.getWidth() / aspectRatio);
            Log.i(TAG, "Expect height " + expectedHeight);
            factorY = 1.0f * expectedHeight / mSurfaceView.getHeight();
        }

        Log.i(TAG, "Calculate scale factor x " + factorX + " y " + factorY);

        mSurfaceView.setScaleX(factorX);
        mSurfaceView.setScaleY(factorY);
    }

    @Override
    public void onOrientationUpdate(int oldOrientation, int newOrientation) {
        Log.i(TAG, "onOrientationUpdate from " + oldOrientation + " to " + newOrientation);
        switch (newOrientation) {
            case IOrientationUpdateListener.ORIENTATION_0:
            case IOrientationUpdateListener.ORIENTATION_180:
                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                break;
            case IOrientationUpdateListener.ORIENTATION_90:
                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                break;
            default:
                Log.i(TAG, "Unknown new orientation: " + newOrientation);
                break;
        }
    }
}