package com.example.aspdemo;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.wuying.aspsdk.aspengine.ui.StreamView;

import java.util.List;

public class DesktopListActivity extends AppCompatActivity {
    private static final String TAG = "DesktopListActivity";

    private ProgressBar mLoadingProgressBar;
    private ListView mDesktopListView;

    class ListAdapter extends ArrayAdapter<ADHelper.Desktop> {

        public ListAdapter(Context context, int resource, List<ADHelper.Desktop> items) {
            super(context, resource, items);
        }

        @Override
        public View getView(int position, View view, ViewGroup parent) {
            View v = view;
            if (v == null) {
                v = LayoutInflater.from(DesktopListActivity.this).inflate(R.layout.desktop_list_item, null);
            }

            ADHelper.Desktop item = getItem(position);

            ImageView imageView = v.findViewById(R.id.desktop_image);
            if (item.mOSType.equals("Windows")) {
                imageView.setImageResource(R.drawable.windows);
            }
            TextView nameView = v.findViewById(R.id.desktop_name);
            TextView versionView = v.findViewById(R.id.desktop_status);

            nameView.setText(item.mName);
            versionView.setText(item.mStatus);

            return v;
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_desktop_list);

        Bundle b = getIntent().getExtras();
        String user = b.getString("user");
        setTitle(user);

        mLoadingProgressBar = findViewById(R.id.loading);
        mDesktopListView = findViewById(R.id.desktop_list_view);

        mDesktopListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                mLoadingProgressBar.setVisibility(View.VISIBLE);
                ADHelper.Desktop desktop = (ADHelper.Desktop)adapterView.getItemAtPosition(i);
                final String desktopId = desktop.mId;
                Log.i(TAG, "open desktop " + desktopId);
                ADHelper.getInstance().getConnectionTicket(DesktopListActivity.this, desktop, new ADHelper.IADConnectionTicketResponse() {
                    @Override
                    public void onSuccess(String ticket) {
                        mLoadingProgressBar.setVisibility(View.GONE);
                        Bundle bundle = new Bundle();
                        bundle.putString(StreamView.CONFIG_DESKTOP_ID, desktopId);
                        bundle.putString(StreamView.CONFIG_CONNECTION_TICKET, ticket);
                        bundle.putString(StreamView.CONFIG_CA_FILE_PATH, CAFileHelper.getCAFilePath(DesktopListActivity.this));
                        bundle.putBoolean(StreamView.CONFIG_PREFER_RTC_TRANSPORT, true);
                        bundle.putBoolean(StreamView.CONFIG_ENABLE_VDAGENT_CHECK, true);
                        bundle.putBoolean(StreamView.CONFIG_ENABLE_STATISTICS, true);
//                        Intent intent = new Intent(
//                                DesktopListActivity.this, CustomizedPlayerDemoActivity.class);
                        Intent intent = new Intent(
                                DesktopListActivity.this, StreamViewDemoActivity.class);
                        intent.putExtras(bundle);
                        intent.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                        startActivity(intent);
                    }

                    @Override
                    public void onFailed(long errorCode) {
                        mLoadingProgressBar.setVisibility(View.GONE);
                        Log.i(TAG, "Failed to get ticket from remote server!");
                        Toast.makeText(DesktopListActivity.this,
                                "Failed to get ticket from remote server!", Toast.LENGTH_LONG);
                    }
                });
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();

        mLoadingProgressBar.setVisibility(View.VISIBLE);

        ADHelper.getInstance().getDesktopList(this, new ADHelper.IADDesktopListResponse() {
            @Override
            public void onSuccess(List<ADHelper.Desktop> desktopList) {
                mLoadingProgressBar.setVisibility(View.GONE);
                mDesktopListView.setAdapter(new ListAdapter(DesktopListActivity.this, R.layout.desktop_list_item, desktopList));
            }

            @Override
            public void onFailed(long errorCode) {
                Toast.makeText(DesktopListActivity.this, "Error: " + errorCode, Toast.LENGTH_LONG);
            }
        });
    }
}