package com.example.aspdemo;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.app.Activity;
import android.content.Context;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewPropertyAnimator;
import android.widget.PopupMenu;
import android.widget.RelativeLayout;

import com.aliyun.wuying.aspsdk.aspengine.ui.StreamView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

public class MenuBar extends RelativeLayout {
    private static final String TAG = "MenuBar";

    private final double RESOLUTION_SCALE_FACTOR_HIGH = 1.0f;
    private final double RESOLUTION_SCALE_FACTOR_MIDDLE = 0.7f;
    private final double RESOLUTION_SCALE_FACTOR_LOW = 0.4f;

    private StreamView mStreamView = null;

    private Context mContext = null;
    private FloatingActionButton mRoot = null;
    private FloatingActionButton mStopStreamButton = null;
    private FloatingActionButton mSetResolutionButton = null;
    private FloatingActionButton mSetMouseModeButton = null;
    private RelativeLayout mLevel1Menu = null;
    private RelativeLayout mLevel2Menu = null;
    private View mResolutionList = null;
    private PopupMenu mResolutionPopMenu = null;

    private int mScreenWidth = 0;
    private int mScreenHeight = 0;

    private float mMenuMargin = 0f;

    private boolean mExpanded = false;

    public MenuBar(Context context) {
        super(context);

        initView(context);
    }

    public MenuBar(Context context, AttributeSet attrs) {
        super(context, attrs);

        initView(context);
    }

    public MenuBar(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        initView(context);
    }

    public MenuBar(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);

        initView(context);
    }

    private void initView(Context context) {
        LayoutInflater inflater = LayoutInflater.from(context);
        if (inflater == null) {
            Log.e(TAG, "Unable to init view due to null reference of LayoutInflater!!!");
        } else {
            mContext = context;
            View v = inflater.inflate(R.layout.menu_bar, this, true);

            getScreenSize();

            mRoot = v.findViewById(R.id.menu_top_entry);
            mStopStreamButton = v.findViewById(R.id.stop_stream);
            mSetResolutionButton = v.findViewById(R.id.set_display_resolution);
            mSetMouseModeButton = v.findViewById(R.id.set_mouse_mode);
            mLevel1Menu = v.findViewById(R.id.menu_bar_level_1);
            //mLevel2Menu = v.findViewById(R.id.menu_bar_level_2);
            //mResolutionList = v.findViewById(R.id.resolution_list);

            mResolutionPopMenu = new PopupMenu(mContext, mSetResolutionButton, Gravity.TOP);
            mResolutionPopMenu.getMenuInflater().inflate(R.menu.resolution_popup_menu, mResolutionPopMenu.getMenu());
            mResolutionPopMenu.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
                @Override
                public boolean onMenuItemClick(MenuItem menuItem) {
                    int itemId = menuItem.getItemId();
                    double scaleRatio = 1.0f;
                    if (itemId == R.id.resolution_option_high) {
                        scaleRatio = RESOLUTION_SCALE_FACTOR_HIGH;
                    } else if (itemId == R.id.resolution_option_middle) {
                        scaleRatio = RESOLUTION_SCALE_FACTOR_MIDDLE;
                    } else if (itemId == R.id.resolution_option_low) {
                        scaleRatio = RESOLUTION_SCALE_FACTOR_LOW;
                    }
                    int toWidth = (int) (mScreenWidth * scaleRatio);
                    int toHeight = (int) (mScreenHeight * scaleRatio);

                    int toWidthMod4 = toWidth % 4;
                    int toHeightMod4 = toHeight % 4;

                    Log.i(TAG, String.format("set resolution screen size %d|%d to size %d|%d with factor %f, widthMod4 %d heightMod4 %d",
                            mScreenWidth, mScreenHeight, toWidth, toHeight, scaleRatio, toWidthMod4, toHeightMod4));

                    if (toWidth + toWidthMod4 < mScreenWidth) {
                        toWidth += toWidthMod4;
                    }

                    if (toHeight + toHeightMod4 < mScreenHeight) {
                        toHeight += toHeightMod4;
                    }

                    mStreamView.setVideoProfile(toWidth, toHeight, -1, null);

                    return true;
                }
            });

            mMenuMargin = getResources().getDimension(R.dimen.menu_margin_1);
            mRoot.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View view) {
                    if (mExpanded) {
                        collapse();
                    } else {
                        expand();
                    }
                }
            });

            mStopStreamButton.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View view) {
                    if (mStreamView != null) {
                        DialogHelper.getInstance().setState(DialogHelper.State.STATE_DISCONNECTING);
                        mStreamView.dispose();
                    }
                }
            });

            mSetResolutionButton.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View view) {
                    /*
                    if (mResolutionList.getVisibility() == VISIBLE) {
                        collapseLevel2Menu(mResolutionList);
                    } else {
                        expandLevel2Menu(mResolutionList);
                    }*
                     */
                    mResolutionPopMenu.show();

                }
            });

            mSetMouseModeButton.setOnClickListener(new OnClickListener() {
                private boolean mEnableMouseMode = false;

                @Override
                public void onClick(View view) {
                    mEnableMouseMode = !mEnableMouseMode;
                    mStreamView.getASPEngineDelegate().enableMouseMode(mEnableMouseMode);
                }
            });

            Log.i(TAG, "Init view complete.");
        }
    }

    private void getScreenSize() {
        DisplayMetrics displayMetrics = new DisplayMetrics();
        ((Activity)mContext).getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        mScreenWidth = displayMetrics.widthPixels;
        mScreenHeight = displayMetrics.heightPixels;
    }

    private void expandLevel2Menu(View item) {
        item.setVisibility(VISIBLE);
        mLevel2Menu.setVisibility(VISIBLE);
        LayoutParams layoutParams = (LayoutParams)getLayoutParams();
        int currentWidth = getRight() - getLeft();
        currentWidth += item.getWidth();
        layoutParams.width = currentWidth;
        setLayoutParams(layoutParams);

        layoutParams = (LayoutParams)mLevel1Menu.getLayoutParams();
        layoutParams.leftMargin += item.getWidth();
        mLevel1Menu.setLayoutParams(layoutParams);

        layoutParams = (LayoutParams)mLevel2Menu.getLayoutParams();

        requestLayout();
    }

    private void collapseLevel2Menu(View item) {
        item.setVisibility(INVISIBLE);
        mLevel2Menu.setVisibility(INVISIBLE);
        LayoutParams layoutParams = (LayoutParams)getLayoutParams();
        int currentWidth = getRight() - getLeft();
        currentWidth -= item.getWidth();
        layoutParams.width = currentWidth;
        setLayoutParams(layoutParams);

        layoutParams = (LayoutParams)mLevel1Menu.getLayoutParams();
        layoutParams.leftMargin -= item.getWidth();
        mLevel1Menu.setLayoutParams(layoutParams);
        requestLayout();
    }

    public void setStreamView(StreamView streamView) {
        mStreamView = streamView;
    }

    public void expand() {
        if (mExpanded) {
            return;
        }
        Log.i(TAG, "expand");
        mExpanded = true;
        mRoot.setAlpha(1.0f);
        mStopStreamButton.setVisibility(VISIBLE);
        mSetResolutionButton.setVisibility(VISIBLE);
        mSetMouseModeButton.setVisibility(VISIBLE);
        mStopStreamButton.animate().translationY(-mMenuMargin - mRoot.getHeight());
        mSetResolutionButton.animate().translationY(-mMenuMargin - mRoot.getHeight()
                - mMenuMargin - mStopStreamButton.getHeight());
        mSetMouseModeButton.animate().translationY(-mMenuMargin - mRoot.getHeight()
                - mMenuMargin - mStopStreamButton.getHeight()
                - mMenuMargin - mSetResolutionButton.getHeight());
    }

    public void collapse() {
        if (!mExpanded) {
            return;
        }
        Log.i(TAG, "collapse");
        mExpanded = false;
        final ViewPropertyAnimator animator1 = mStopStreamButton.animate().translationY(0);
        animator1.setListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                super.onAnimationEnd(animation);
                mStopStreamButton.setVisibility(GONE);
                animator1.setListener(null);
            }
        });
        final ViewPropertyAnimator animator2 = mSetResolutionButton.animate().translationY(0);
        animator2.setListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                super.onAnimationEnd(animation);
                mSetResolutionButton.setVisibility(GONE);
                animator2.setListener(null);
            }
        });
        final ViewPropertyAnimator animator3 = mSetMouseModeButton.animate().translationY(0);
        animator3.setListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                super.onAnimationEnd(animation);
                mSetMouseModeButton.setVisibility(GONE);
                animator3.setListener(null);
            }
        });
        mRoot.animate().alpha(0.3f);
    }
}
