package com.example.aspdemo;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.media.MediaCodecInfo;
import android.media.MediaCodecList;
import android.media.MediaFormat;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.util.Log;
import android.util.Range;
import android.view.KeyEvent;
import android.view.Window;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

//import com.aliyun.wuying.aspsdk.aspengine.ClipboardData;
import com.aliyun.wuying.aspsdk.aspengine.ClipboardData;
import com.aliyun.wuying.aspsdk.aspengine.IASPEngineListener;
import com.aliyun.wuying.aspsdk.aspengine.IRuntimeStatusInspector;
import com.aliyun.wuying.aspsdk.aspengine.ui.StreamView;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringWriter;

public class StreamViewDemoActivity extends Activity {
    private static final String TAG = "StreamViewDemoActivity";
    private static final int WRITE_REQUEST_CODE = 1;
    private StreamView mStreamView = null;
    private MenuBar mMenuBar = null;
    private Bundle mConfigs = null;
    private boolean mDumpData = false;

    private IASPEngineListener mListener = new IASPEngineListener() {
        @Override
        public void onConnectionSuccess() {
            Log.i(TAG, "onConnectionSuccess");
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTED);

            //mStreamView.setVideoProfile(1280, 800, 30, null);
        }

        @Override
        public void onConnectionFailure(int errorCode, String errorMsg) {
            Log.i(TAG, "onConnectionFailure errorCode:" + errorCode + " errorMsg:" + errorMsg);
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTION_ERROR,
                    String.valueOf(errorCode), errorMsg);
        }

        @Override
        public void onEngineError(int errorCode, String errorMsg) {
            Log.i(TAG, "onEngineError errorCode:" + errorCode + " errorMsg:" + errorMsg);
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_ENGINE_ERROR,
                    String.valueOf(errorCode), errorMsg);
        }

        @Override
        public void onDisconnected(int reason) {
            Log.i(TAG, "onDisconnected reason=" + reason);
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_DISCONNECTED);
        }

        @Override
        public void onReconnect() {
            Log.i(TAG, "onReconnect");
            DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTING);
        }

        @Override
        public void onFirstFrameRendered(long timeCostMS) {
            Log.i(TAG, "onFirstFrameRendered");
        }

        @Override
        public void onPolicyUpdate(String policy) {
            Log.i(TAG, "policy=" + policy);
        }

        @Override
        public void onSessionSuccess() {

        }
    };

    @Override
    public void onConfigurationChanged(@NonNull Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
    }

    private WifiManager.WifiLock mWifiLowLatencyLock = null;
    private WifiManager.WifiLock mWifiHighPerfLock = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);

        setContentView(R.layout.activity_stream_view_demo);

        DialogHelper.getInstance().initialize(this);

        PreferenceManager.setDefaultValues(this, R.xml.root_preferences, false);
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);

        mConfigs = getIntent().getExtras();

        mMenuBar = findViewById(R.id.menu_bar);

        mStreamView = findViewById(R.id.stream_view);
        mStreamView.scaleStreamVideo(pref.getBoolean("fit_video_content", true) ?
                StreamView.ScaleType.FIT_STREAM_CONTENT : StreamView.ScaleType.FILL_STREAM_VIEW);

        if (mDumpData) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                String[] permissions = {
                        Manifest.permission.READ_EXTERNAL_STORAGE,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE
                };
                requestPermissions(permissions, WRITE_REQUEST_CODE);
            }
        }

        //mStreamView.getASPEngineDelegate().setVideoProfile(1798, 984,30, null);
        //mStreamView.getASPEngineDelegate().registerASPEngineListener(mListener);
        mStreamView.getASPEngineDelegate().registerASPEngineListener(mListener);

        mMenuBar.setStreamView(mStreamView);

        mStreamView.start(mConfigs);

        DialogHelper.getInstance().setState(DialogHelper.State.STATE_CONNECTING);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        switch (requestCode) {
            case WRITE_REQUEST_CODE:
                for (int i = 0; i < permissions.length; i++) {
                    Log.i(TAG, "permission " + permissions[i] + " granted: "
                            + (grantResults[i] == PackageManager.PERMISSION_GRANTED));
                }
                break;
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        DialogHelper.getInstance().reset();
        mStreamView.dispose();
        mStreamView = null;
    }

}