package com.example.aspdemo.data;

import android.content.Context;
import com.example.aspdemo.ADHelper;
import com.example.aspdemo.data.model.LoggedInUser;

import java.io.IOException;

/**
 * Class that handles authentication w/ login credentials and retrieves user information.
 */
public class LoginDataSource {
    private static final String TAG = "LoginDataSource";

    private Context mContext;

    static public interface ILoginCallback {
        void onLoginSuccess(Result<LoggedInUser> user);
        void onLoginFailed(Result<LoggedInUser> user);
    }

    public LoginDataSource(Context context) {
        mContext = context;
    }

    public void login(String username, String password, final ILoginCallback loginCallback) {
        final String loginUserName = username;
        ADHelper.getInstance().login(username, password, mContext, new ADHelper.IADLoginResponse() {
            @Override
            public void onSuccess(String response) {
                try {
                    // TODO: handle loggedInUser authentication
                    LoggedInUser fakeUser =
                            new LoggedInUser(
                                    java.util.UUID.randomUUID().toString(),
                                    loginUserName);
                    loginCallback.onLoginSuccess(new Result.Success<>(fakeUser));
                } catch (Exception e) {
                }
            }

            @Override
            public void onFailed(long errorCode) {
                loginCallback.onLoginFailed(new Result.Error(
                        new IOException("Error logging in due to " + errorCode)));
            }
        });
    }

    public void logout() {
        // TODO: revoke authentication
    }
}