package com.example.aspdemo.mediastreamplayer.audio;

import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.util.Log;

import com.aliyun.wuying.aspsdk.aspengine.AFrame;
import com.aliyun.wuying.aspsdk.aspengine.AudioMode;
import com.aliyun.wuying.aspsdk.aspengine.ErrorCode;

class AudioTrackPlayer {
    private static final String TAG = "AudioTrackPlayer";
    private static final long UNSIGNED_INTEGER_MAX = 65535;
    private AudioTrack mTrack;
    private float mVolume = 1;

    public ErrorCode init() {
        Log.i(TAG, "init");
        return ErrorCode.OK;
    }

    public ErrorCode deinit() {
        Log.i(TAG, "deinit");
        if (mTrack != null) {
            mTrack.release();
            mTrack = null;
        }
        return ErrorCode.OK;
    }

    public ErrorCode startAudioPlayback() {
        Log.i(TAG, "startAudioPlayback");
        if (mTrack != null) {
            mTrack.play();
        }
        return ErrorCode.OK;
    }

    public ErrorCode stopAudioPlayback() {
        Log.i(TAG, "stopAudioPlayback");
        if (mTrack != null) {
            mTrack.pause();
        }
        return ErrorCode.OK;
    }

    public ErrorCode pushAudioPlaybackFrame(AFrame pbData) {
        Log.i(TAG, "pushAudioPlaybackFrame " + pbData.toString());

        // TODO: OPUS - need to implement the decoder
        if (pbData.mode == AudioMode.OPUS) {
            return ErrorCode.OK;
        }

        if (mTrack == null) {
            int bufsize = AudioTrack.getMinBufferSize(
                    pbData.frequency,
                    AudioFormat.CHANNEL_OUT_STEREO,
                    AudioFormat.ENCODING_PCM_16BIT
            );

            mTrack = new AudioTrack(
                    AudioManager.STREAM_MUSIC, pbData.frequency,
                    AudioFormat.CHANNEL_OUT_STEREO,
                    AudioFormat.ENCODING_PCM_16BIT,
                    bufsize,
                    AudioTrack.MODE_STREAM
            );

            mTrack.play();
            mTrack.setVolume(mVolume);
        }

        if (mTrack != null) {
            int ret = mTrack.write(pbData.frame, 0, pbData.size);
            if (ret < 0) {
                Log.e(TAG, "Failed to write audio data due to " + ret);
                return ErrorCode.ERROR;
            }
        }

        return ErrorCode.OK;
    }

    public ErrorCode updateAudioPlaybackVol(int volume) {
        Log.i(TAG, "updateAudioPlaybackVol " + volume);
        mVolume = (float)volume / UNSIGNED_INTEGER_MAX;

        if (mTrack != null) {
            int ret = mTrack.setVolume(mVolume);
            if (ret < 0) {
                Log.e(TAG, "Failed to set audio volume due to " + ret);
                return ErrorCode.ERROR;
            }
        }

        return  ErrorCode.OK;
    }

    public ErrorCode updateAudioPlaybackMute(int mute) {
        Log.i(TAG, "updateAudioPlaybackMute " + mute);

        if (mTrack != null) {
            int ret = mTrack.setVolume(mute != 0 ? 0 : mVolume);
            if (ret < 0) {
                Log.e(TAG, "Failed to set audio mute due to " + ret);
                return ErrorCode.ERROR;
            }
        }

        return ErrorCode.OK;
    }

}
