package com.example.aspdemo.mediastreamplayer.video;

import android.graphics.SurfaceTexture;
import android.opengl.EGL14;
import android.opengl.GLES20;
import android.util.Log;
import android.view.Surface;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLContext;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.egl.EGLSurface;

class GLRenderer implements SurfaceTexture.OnFrameAvailableListener {
    private static final String TAG = "GLRenderer";

    class RenderTexture {
        int mTextureID;
        SurfaceTexture mSurfTexture;
        Surface mSurface;
    }
    private final int NUM_TEXTURES = 1;
    private int mTextures[] = new int[NUM_TEXTURES];
    private RenderTexture mTexture;
    private STextureRender mTextureRender = new STextureRender();
    private Thread mWorker = null;
    private boolean mStop = false;
    private boolean mFrameAvailable = false;

    @Override
    public void onFrameAvailable(SurfaceTexture surfaceTexture) {
        Log.i(TAG, "frame available surftex: " + surfaceTexture);
        synchronized (GLRenderer.this) {
            mFrameAvailable = true;
            notify();
        }
    }

    public void stop() {
        synchronized (this) {
            mStop = true;
            notify();
        }

        try {
            mWorker.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    private EGL10 egl;
    private EGLDisplay display;
    private EGLSurface eglSurface;
    private EGLContext eglContext;

    public GLRenderer(Surface surface) {

        egl = (EGL10)EGLContext.getEGL();
        display = egl.eglGetDisplay(EGL10.EGL_DEFAULT_DISPLAY);
        egl.eglInitialize(display, null);

        int[] attribList = {
                EGL10.EGL_RED_SIZE, 8,
                EGL10.EGL_GREEN_SIZE, 8,
                EGL10.EGL_BLUE_SIZE, 8,
                EGL10.EGL_ALPHA_SIZE, 8,
                EGL10.EGL_RENDERABLE_TYPE, EGL14.EGL_OPENGL_ES2_BIT,
                EGL10.EGL_SURFACE_TYPE, EGL10.EGL_WINDOW_BIT,
                EGL10.EGL_DEPTH_SIZE, 8,
                EGL10.EGL_STENCIL_SIZE, 8,
                EGL10.EGL_SAMPLE_BUFFERS, 1,
                EGL10.EGL_SAMPLES, 2,
                EGL10.EGL_NONE, 0,
                EGL10.EGL_NONE
        };

        EGLConfig[] configs = new EGLConfig[1];
        int[] numConfigs = new int[1];
        egl.eglChooseConfig(display, attribList, configs, configs.length, numConfigs);
        EGLConfig config = configs[0];

        eglSurface = egl.eglCreateWindowSurface(display, config, surface, new int[]{
                EGL10.EGL_NONE
        });

        eglContext = egl.eglCreateContext(display, config, EGL10.EGL_NO_CONTEXT, new int[]{
                EGL14.EGL_CONTEXT_CLIENT_VERSION, 2,
                EGL10.EGL_NONE
        });

        mWorker = new Thread(new Runnable() {
            private void prepareContext() {
                egl.eglMakeCurrent(display, eglSurface, eglSurface, eglContext);
            }

            private void prepareTexture() {
                synchronized (GLRenderer.this) {
                    GLES20.glGenTextures(NUM_TEXTURES, mTextures, 0);

                    mTexture = new RenderTexture();
                    mTexture.mSurfTexture = new SurfaceTexture(mTextures[0]);
                    mTexture.mSurface = new Surface(mTexture.mSurfTexture);
                    mTexture.mTextureID = mTextures[0];
                    mTexture.mSurfTexture.setOnFrameAvailableListener(GLRenderer.this);

                    //mTexturesInUse.put(mTextures[0], renderTexture);
                    //mAvailableTextures.add(renderTexture);

                    mTextureRender.surfaceCreated();

                    GLRenderer.this.notify();
                }

                Log.i(TAG, "gl surfaceCreated tex " + mTexture.mSurfTexture);
            }

            @Override
            public void run() {
                prepareContext();

                prepareTexture();

                while (!mStop) {
                    synchronized (GLRenderer.this) {
                        while (!mFrameAvailable && !mStop) {
                            try {
                                GLRenderer.this.wait();
                            } catch (InterruptedException e) {
                                e.printStackTrace();
                                break;
                            }
                        }

                        if (mFrameAvailable) {
                            mTexture.mSurfTexture.updateTexImage();
                            mTextureRender.drawFrame(mTexture.mSurfTexture, mTexture.mTextureID, false);
                            mFrameAvailable = false;

                            egl.eglSwapBuffers(display, eglSurface);
                            VideoStreamEventHandler.getInstance().onVideoFrameRendered();
                        }
                    }
                }

                egl.eglDestroySurface(display, eglSurface);
                egl.eglMakeCurrent(display, EGL10.EGL_NO_SURFACE, EGL10.EGL_NO_SURFACE, EGL10.EGL_NO_CONTEXT);
                egl.eglDestroyContext(display, eglContext);
                egl.eglTerminate(display);
            }
        });

        mWorker.start();
    }

    public synchronized Surface getSurface() {
        while (mTexture == null) {
            try {
                wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        return mTexture.mSurface;
    }
}
