package com.example.aspdemo.mediastreamplayer.video;

import com.aliyun.wuying.aspsdk.aspengine.Event;
import com.aliyun.wuying.aspsdk.aspengine.EventHandler;
import com.aliyun.wuying.aspsdk.aspengine.EventType;
import com.aliyun.wuying.aspsdk.aspengine.VDecodePerfInfo;
import com.aliyun.wuying.aspsdk.aspengine.VFrame;
import com.aliyun.wuying.aspsdk.aspengine.VRenderPerfInfo;

import java.util.ArrayDeque;
import java.util.Queue;

public class VideoStreamEventHandler {
    private static final String TAG = "VideoStreamHandler";
    private static VideoStreamEventHandler mInstance = new VideoStreamEventHandler();

    private EventHandler mHandler;

    private Queue<VFrame> mVideoFrame = new ArrayDeque<>();
    private boolean mEnabled = false;

    private VideoStreamEventHandler() {
    }

    public static VideoStreamEventHandler getInstance() {
        return mInstance;
    }

    public void setEnabled(boolean enabled) {
        mEnabled = enabled;
    }

    public synchronized void reset(EventHandler handler) {
        mHandler = handler;
        mVideoFrame.clear();
    }

    public synchronized void onReceiveVideoFrame(VFrame frame) {
        mVideoFrame.add(frame);
    }

    public synchronized void onVideoFrameRendered() {
        VFrame frame = mVideoFrame.remove();
        if (mEnabled && mHandler != null) {
            Event event = new Event();
            event.type = EventType.RENDER_PERF_INFO;
            event.decodePerfInfo = new VDecodePerfInfo();
            event.renderPerfInfo = new VRenderPerfInfo();
            event.renderPerfInfo.frameId = frame.frameId;
            event.renderPerfInfo.sessionId = frame.sessionId;
            mHandler.callback(event);
        }
    }
}
