package com.example.aspdemo.mediastreamplayer.video;

import android.util.Log;
import android.view.Surface;

import com.aliyun.wuying.aspsdk.aspengine.ErrorCode;
import com.aliyun.wuying.aspsdk.aspengine.EventHandler;
import com.aliyun.wuying.aspsdk.aspengine.MediaStreamPlayer;
import com.aliyun.wuying.aspsdk.aspengine.VFrame;
import com.aliyun.wuying.aspsdk.aspengine.VProfile;

import java.util.HashMap;

public class VideoStreamHandler implements MediaStreamPlayer.IVideoStreamHandler {
    private static final String TAG = "VideoStreamHandler";

    private MediaCodecVideoDecoder mDecoder = null;
    private GLRenderer mRenderer = null;
    private HashMap<Integer, VProfile> mVideoProfiles = new HashMap<>();

    @Override
    public void setEventHandler(EventHandler handler) {
        Log.i(TAG, "setEventHandler handler " + handler);
        VideoStreamEventHandler.getInstance().reset(handler);
    }

    @Override
    public ErrorCode addVideoTrack(int trackId, VProfile profile) {
        Log.i(TAG, "addVideoTrack track " + trackId + " " + profile.toString());
        mVideoProfiles.put(trackId, profile);
        mDecoder = new MediaCodecVideoDecoder();
        return mDecoder.setVideoProfile(profile) ? ErrorCode.OK : ErrorCode.ERROR;
    }

    @Override
    public ErrorCode setVideoSurface(int trackId, Surface surface) {
        Log.i(TAG, "setVideoSurface track " + trackId + " " + surface.toString());

        if (mRenderer != null) {
            mRenderer.stop();
        }

        if (surface != null) {
            mRenderer = new GLRenderer(surface);
        }
        return mDecoder.setSurface(mRenderer.getSurface()) ? ErrorCode.OK : ErrorCode.ERROR;
    }

    @Override
    public ErrorCode playVideo(int trackId) {
        Log.i(TAG, "playVideo track " + trackId);
        return mDecoder.start() ? ErrorCode.OK : ErrorCode.ERROR;
    }

    @Override
    public ErrorCode removeVideoTrack(int trackId) {
        Log.i(TAG, "removeVideoTrack track " + trackId);
        mVideoProfiles.remove(trackId);
        return mDecoder.stop() ? ErrorCode.OK : ErrorCode.ERROR;
    }

    @Override
    public ErrorCode pauseVideoTrack(int trackId) {
        Log.i(TAG, "pauseVideoTrack track " + trackId);
        return ErrorCode.OK;
    }

    @Override
    public ErrorCode resumeVideoTrack(int trackId) {
        Log.i(TAG, "resumeVideoTrack track " + trackId);
        return ErrorCode.OK;
    }

    @Override
    public ErrorCode pushVideoFrame(int trackId, VFrame frame) {
        Log.i(TAG, "pushVideoFrame track " + trackId + " " + frame.toString());
        VideoStreamEventHandler.getInstance().onReceiveVideoFrame(frame);
        return mDecoder.handleInputFrame(frame) ? ErrorCode.OK : ErrorCode.ERROR;
    }

    @Override
    public HashMap<Integer, VProfile> getVideoTracks() {
        Log.i(TAG, "getVideoTracks");
        return mVideoProfiles;
    }

    @Override
    public ErrorCode release() {
        Log.i(TAG, "release");
        return mDecoder.stop() ? ErrorCode.OK : ErrorCode.ERROR;
    }
}
